package AliGROOVE_module;

use strict    ;
use warnings  ;
use Tie::File ;
use Fcntl     ;

#use Cwd qw(realpath)			;
#	use File::Basename qw(dirname)	;
#	use lib dirname(realpath($0))	;
#converts different line feeds in open process
use open IN => ":raw", OUT => ":raw" ;


#generated by Bernhard Misof, 15th April    2007

#updated   by Bernhard Misof,  4th July     2007
#updated   by Bernhard Misof, 26th August   2007
#updated   by Bernhard Misof, 10th November 2007
#updated   by Bernhard Misof, 22nd December 2007
#updated   by Bernhard Misof,  2nd January  2008
#updated   by Bernhard Misof, 10th February 2008
#updated   by Bernhard Misof, 16th February 2008
#updated   by Bernhard Misof,  4th March    2008
#updated   by Bernhard Misof,  7th March    2008
#updated   by Bernhard Misof, 11th March    2008
#updated   by Bernhard Misof, 26th March    2008
#updated   by Bernhard Misof,  2nd April    2008
#updated   by Bernhard Misof, 23nd June     2008
#updated   by Bernhard Misof,  2nd September 2008
#updated   by Bernhard Misof, 18th November  2008  => little bug if taxon names were mixed with pure numerics and alphanumerics
#updated   by Patrick Kück, 17th October 2013


sub get_options {
	
	#no warnings;
	
	my (
		$file		,
		$window	,
		$option	,
		$indels	, 
		$matrix	,
		%options	,
		$phylotree	,
	) ;

our $USAGE = do {

<<USAGE;

USAGE   : AliGROOVE.pl -i infile.fas
                             
options	: [-N] 
	  [-w [#]] 	
	  [-z [treefile.tre]] 
	  [-h] 
	  
defaults: without -N      : gaps treated as fifth character, 
			    will be ignored for 'aa' data
	  
	  without -w #    : window size is 6
	  
	  with -z treefile: guided topology for tree tagging
	 
	  with -h         : help option  

random input order of options allowed

OUTPUT:
2 output files without -z option:
	  
	-AliGROOVE_seqsim_matrix_infile.svg
	 (colour coded sequence pairwise similarity matrix)
	  
	-AliGROOVE_seqsim_matrix_infile.txt
	 (Pairwise sequence similarity matrix scores)
	
+2 output files with -z option:
	
	-AliGROOVE_tagged_info_treefile.txt
	 (summarized branch reliability tagging information)
	
	-AliGROOVE_tagged_tree_treename.svg
	 (tagged branch reliabilities specified via -z option)
------------------------------------------------------------
------------------------------------------------------------


	  
USAGE

};

die $USAGE if ! @ARGV ;

our $HELP = do {

<<HELP;

usage: AliGROOVE.pl -i infile 

options	: [-N] 
	  [-w [#]] 	
	  [-z [treefile]] 
	  [-h] 		             

for detailed help on options type : 

AliGROOVE.pl help option 

e.g. AliGROOVE.pl help -i 
  or AliGROOVE.pl help -z

     AliGROOVE.pl help -output
     AliGROOVE.pl help -scoring
     AliGROOVE.pl help -commands

to get information on output and scoring


HELP

};		

		for (
				$file		,
				$window	,
				$option	,
				$indels	, 
				$phylotree	,
				$matrix
		){ $_ = '-' }
		
		my @INPUT = @ARGV ;
		
		if ( grep /help/ , @INPUT ) { 
			
			no warnings ;
			
			die $HELP if   ($INPUT[0] =~ /^help$/ and ! defined $INPUT[1]) ;
			die $HELP if   ($INPUT[0] !~ /^help$/ ) ;
			
			if ( ($INPUT[1]	!~ / (^\-w(\s*\d*))$	|
								(^\-i$				)	|
								(^\-h$				)	|
								(^\-N$				)	|
								(^\-z$				)	|
								(^\-output$			)	|
								(^\-scoring$		)	|
								(^\-commands$		)	/x)  
			){ die "don't know what you want! use e.g. help -commands\n" }
				
				help ( $INPUT[1] ) if $INPUT[0] =~ /^help$/ and $INPUT[1] =~ /^-.+$/ ;
				
				exit
		}
		
		for ( @ARGV ) { die "\nUnknown option: $_"."\n\n".$USAGE if /^-[^BPMirwNtlsohecz]/ } ;
		
		
		my $dec_opts = 'irwNosehltz';
		
		$options{$_} = 0  for ( split "",$dec_opts ) ;
		
		die "semicolon not allowed in input options!\n" if map { /\:|;/ } @ARGV ;
	
=pod

If infile is not present, ALiGROOVE stops!

=cut
 
		
		PROTEINMATRIX: {
		
		     no warnings ;		
		
		   ( $matrix ) = grep /^-((PAM250|PAM500|MATCH|BLOSUM62))$ /xi , @INPUT ;
		     $matrix   =~ s/-// ;
					
				}
		
		map { s/^-(.)(.*)$/--$1\:$2/ } @INPUT ;
		
		@INPUT = ( split "--",( join"",@INPUT ) ) ;
		
		shift @INPUT;
		
		my $input_error = do { "don't know what you want! use e.g. help -commands\n" } ;
		
		for ( @INPUT ) {
			
			if ( /^w/ ) { die $input_error if !/ ^w\:\d*$    /x }
			if ( /^h/ ) { die $input_error if !/ ^h\:$       /x }
			if ( /^N/ ) { die $input_error if !/ ^N\:$       /x }
		}
		
		
		for ( @INPUT ) { die "use blanks between options!\n" if !/\:/ }
		
		for ( @INPUT ) {
			
			my ( $key, $value )	= split ":" ;
			
			$options{$key}		= $value			if ( exists $options{$key} and $value =~ /./ );
			$options{$key}		= 1					if ( exists $options{$key} and $value !~ /./ );
			$options{'matrix'}		= uc( $matrix )	if defined $matrix;
		}
		
		INPUT: {
			
			$file		=	$options{i}		if	$options{i}	=~ /\w+/											;
			
			$window	=	$options{w}		if	$options{w}	>  1												;
			$window	=	6					if	$options{w}	<= 1												;
			
			$option	=	qw(N)				if	$options{N}	== 1												;
			$indels	=	'ambiguous'			if	$options{N}	== 1												;
			$indels	=	'characters'		if	$options{N}	== 0												;
			
			$phylotree	=	$options{z}		if	$options{z}	=~ /\w+/											;
			
			$matrix	=	'BLOSUM62'																					;
			$matrix	=	$options{matrix}	if	defined $options{matrix}											;
			
			die $HELP							if	$options{h}	== 1												;
		}
	
		
		$window = AliGROOVE_module::even_numbered_window ( $window ) ;
	
	return  (
			$file		,
			$window	,
			$option	,
			$indels	, 
			$phylotree	,
			$matrix	
		)
		
	}


sub even_numbered_window {
		
		my ( $window ) = @_ ;
		
		my $window2 = $window / 2 ;
	        
		$window += 1 if $window2 =~ /\./ ;

		return $window
		
	}



sub help {
	
		use Text::Wrap qw ($columns &wrap);

		$columns = 70 ;

		my ( $option ) = @_ ;
		
		$option eq '-w' and do {
					
				my @text = (
								'-w option ...','specifies dimension of the sliding window, default is w = 6. Window size below 4 do not make much sense since',
								'error rates for miscalling randomness and non-randomness will be much to high. Only window dimensions of even numbered size ',
								'will be accepted, if an uneven number is called by the user it will be changed to the next larger even number.','Larger window',
								'dimensions will make the profiling less sensitive to small alignment sections of randomness.'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-i' and do {
			
				my @text = (
								'-i option ...','give the full name of the input file including extension. Input file must be in same folder as AliGROOVE.',
								'AliGROOVE currently accepts files in FASTA format, in simple ASCII text format. Avoid formating input files with text editors',
								'like MSWord or something comparable. The first line break is interpreted',
								' as taxon name separator following the FASTA file convention. Taxon names are allowed to contain alphanumeric signs and underscores.',
								'Sequences are allowed to contain nucleotide or amino acid coding signs, ?, indels and ambiguity characters. AliGROOVE estimates data type from ',
								'frequencies of A, C, G, T|U. Only if estimates are consistant among sequencies, AliGROOVE continues with the process using the appropriate',
								'scoring function.','RNA sequences will be recoded to DNA sequences. There is no restriction on sequence number and sequence length.','For more',
								' information on reading the input data consult the manual.'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-z' and do {
			
				my @text = (
								'-z option ...','Defines a guiding tree in NEWICK format to tag potentially unreliable relationships in the defined topology.',
								'AliGROOVE tags terminal branches with the mean pairwise similarity score between the terminal taxon and all other taxa ',
								'To tag internal nodes, AliGROOVE calculates the mean similarity score from all pairwise comparisons across this node.',
								'The calculation of the mean pairwise similarity score treats all pairwise comparisons as independent replicates.',
								'Taxon names have to be identic between the sequence input file and the treefile. Calculated reliabilities of single branches are shown coloured.',
								'Red indicates that ambiguously aligned sequence positions dominate between taxa connected by the respective branch while blue indicates the opposite.',
								'The more positive or negative the mean similarity score, the darker the corresponding colour.',
				);
					
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-N' and do {
			
				my @text = (
								'-N option ...','without invoking the -N option indels are treated as 5th character. With the -N option indels are treated as ',
								'ambiguous characters. This option is ignored for amino acid sequences.'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-output' and do {
				
				my @text = (
								'output ...    ','AliGROOVE prints two output files whithout additional tree specification and four output files if a topology has ',
								'been specified via the -z option. The output files are printed in the same folder in which the input data is stored: ',
								'AliGROOVE seqsim matrix infile.svg -> colour coded pairwise sequence similarity matrix; AliGROOVE seqsim matrix infile.txt -> pairwise sequence similarity matrix scores; ',
								'AliGROOVE tagged info treefile.txt (with -z option) -> summarized branch reliability tagging information of a given treefile; ',
								'AliGROOVE tagged tree treefile.svg (with -z option) -> tagged branch reliabilities of a given treefile'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-scoring' and do {
				
				my @text = (
								'scoring ...   ','For nucleotide sequences AliGROOVE uses a simple match/mismatch scoring matrix to calculate the observed window ',
								'and resampled window scores. If ambiguities are present, AliGROOVE makes an optimistic estimate with a reduced match score according',
								'to degeneracy of ambiguities. Indels are either scored as 5th character or ambiguities.',
								'For aminoacid sequences a BLOSUM62 matrix is used as default amino acid substitution matrix. Further, optionally available, amino acid substitution matrices are PAM250 and PAM500. ',
								'Indels are scored as strongly penalized mismatches when matching and not penalized if matching an amino acid. ',
								'This scoring induces the effect, that if indels dominate in sections, these sections will be scored negatively in consensus profiles',
								', but if aminoacids dominate the section, indels will not have a negative effect.','this scoring was set to account for missing data,',
								'typical in concatenated EST or phylogenomic data.'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
		$option eq '-commands' and do {
				
				my @text = (
								'commands ...  ','Input must contain an infile, all other options are not mandatory. Order of options in command line does not ',
								'matter. You can use e.g. AliGROOVE.pl -i infile -z treefile -N or AliGROOVE.pl -N -i infile -z treefile . All options must proceed a -',
								'sign e.g. -i or -z etc.. Specifications of options like tree file or window size must directly ',
								'follow option invokation with blanks inbetween. If you use no option, ALiGROOVE will switch to defaults, whereas',
								'indels are treated as 5th character with window size of 6.'
				);
				
				print			"\nhelp called for $option .... \n\n" ;
				print		wrap("","              ",@text), "\n\n" ;
				
				exit
		};
		
	}




#_______________________________________________________________________________________

sub get_scoring {
			my ( $type, $matrix ) = @_;
			my %SCORING;

			die "sequence type not known!\n"                                       if $type   !~ /(nt|aa)/;
			die "scoring matrix not implemented\nuse BLOSUM62, PAM250, PAM500 or MATCH\n" if $matrix !~ /(BLOSUM62|PAM250|PAM500|MATCH)/; 

			$matrix    = uc ( $matrix ) ;

			my $check  = sub {
					my ($arr1,$arr2) = @_;
					my (%hash2,@overlap);

					$hash2{$_} = 1 for (@$arr2);
					for (0..$#$arr1){push @overlap, $arr1->[$_] if (defined $hash2{$arr1->[$_]})};
	
					my $hit = @overlap;
					return $hit;
					};

			#creates nuleotide scoring function, includes ambiguities

			$type eq "nt" and do {

				my @nucleotides = qw ( A C G T - AC AG AT CG CT GT ACG ACT CGT AGT ACGT A- C- G- T- AC- AG- AT- CG- CT- GT- ACG- ACT- CGT- AGT- ACGT-);
				my %ambiguities =    ('A-'=>'A-','C-'=>'C-','G-'=>'G-','T-'=>'T-','AC'=>'M','AG'=>'R','AT'=>'W','CG'=>'S','CT'=>'Y','GT'=>'K','ACG'=>'V',
						      'ACT'=>'H','CGT'=>'B','AGT'=>'D','ACGT'=>'N','AC-'=>'M-','AG-'=>'R-','AT-'=>'W-','CG-'=>'S-','CT-'=>'Y-','GT-'=>'K-',
						      'ACG-'=>'V-','ACT-'=>'H-','CGT-'=>'B-','AGT-'=>'D-','ACGT-'=>'N-');
				for my $i (@nucleotides){
				    my @i       = split"",$i;#print $i,"\t";
				    my $recode1 = defined $ambiguities{$i} ? $ambiguities{$i} : $i;		
	
					for my $j (@nucleotides){
					    my @j       = split"",$j;#print $j,"\n";exit;
					    my $recode2 = defined $ambiguities{$j} ? $ambiguities{$j} : $j;
				            my $overlap = $check->(\@i,\@j);
				            my $max     = @i > @j ? @i : @j;#print $max,"\n";exit;
	    	       	    
					    $SCORING{$recode1.$recode2} = $overlap > 0 ? 1/$max : -1 if $max < 4; 
					    $SCORING{$recode1.$recode2} = $overlap > 0 ? 0      : -1 if $max > 3;
						
						}
					}

				};#end of do nucleotides!
				
			
			
			$type eq "aa" and do {			
			
				#  Blosum62 source: NCBI
				#  Matrix made by matblas from blosum62.iij
				#  * column uses minimum score
				#  BLOSUM Clustered Scoring Matrix in 1/2 Bit Units
				#  Blocks Database = /data/blocks_5.0/blocks.dat
				#  Cluster Percentage: >= 62
				#  Entropy =   0.6979, Expected =  -0.5209
				
				my @BLOSUM62 =	(
				
						[ 4,-1,-2,-2, 0,-1,-1, 0,-2,-1,-1,-1,-1,-2,-1, 1, 0,-3,-2, 0,-2,-1, 0,-4 ],
						[-1, 5, 0,-2,-3, 1, 0,-2, 0,-3,-2, 2,-1,-3,-2,-1,-1,-3,-2,-3,-1, 0,-1,-4 ],
						[-2, 0, 6, 1,-3, 0, 0, 0, 1,-3,-3, 0,-2,-3,-2, 1, 0,-4,-2,-3, 3, 0,-1,-4 ],
						[-2,-2, 1, 6,-3, 0, 2,-1,-1,-3,-4,-1,-3,-3,-1, 0,-1,-4,-3,-3, 4, 1,-1,-4 ],
						[ 0,-3,-3,-3, 9,-3,-4,-3,-3,-1,-1,-3,-1,-2,-3,-1,-1,-2,-2,-1,-3,-3,-2,-4 ],
						[-1, 1, 0, 0,-3, 5, 2,-2, 0,-3,-2, 1, 0,-3,-1, 0,-1,-2,-1,-2, 0, 3,-1,-4 ],
						[-1, 0, 0, 2,-4, 2, 5,-2, 0,-3,-3, 1,-2,-3,-1, 0,-1,-3,-2,-2, 1, 4,-1,-4 ],
						[ 0,-2, 0,-1,-3,-2,-2, 6,-2,-4,-4,-2,-3,-3,-2, 0,-2,-2,-3,-3,-1,-2,-1,-4 ],
						[-2, 0, 1,-1,-3, 0, 0,-2, 8,-3,-3,-1,-2,-1,-2,-1,-2,-2, 2,-3, 0, 0,-1,-4 ],
						[-1,-3,-3,-3,-1,-3,-3,-4,-3, 4, 2,-3, 1, 0,-3,-2,-1,-3,-1, 3,-3,-3,-1,-4 ],
						[-1,-2,-3,-4,-1,-2,-3,-4,-3, 2, 4,-2, 2, 0,-3,-2,-1,-2,-1, 1,-4,-3,-1,-4 ],
						[-1, 2, 0,-1,-3, 1, 1,-2,-1,-3,-2, 5,-1,-3,-1, 0,-1,-3,-2,-2, 0, 1,-1,-4 ],
						[-1,-1,-2,-3,-1, 0,-2,-3,-2, 1, 2,-1, 5, 0,-2,-1,-1,-1,-1, 1,-3,-1,-1,-4 ],
						[-2,-3,-3,-3,-2,-3,-3,-3,-1, 0, 0,-3, 0, 6,-4,-2,-2, 1, 3,-1,-3,-3,-1,-4 ],
						[-1,-2,-2,-1,-3,-1,-1,-2,-2,-3,-3,-1,-2,-4, 7,-1,-1,-4,-3,-2,-2,-1,-2,-4 ],
						[ 1,-1, 1, 0,-1, 0, 0, 0,-1,-2,-2, 0,-1,-2,-1, 4, 1,-3,-2,-2, 0, 0, 0,-4 ],
						[ 0,-1, 0,-1,-1,-1,-1,-2,-2,-1,-1,-1,-1,-2,-1, 1, 5,-2,-2, 0,-1,-1, 0,-4 ],
						[-3,-3,-4,-4,-2,-2,-3,-2,-2,-3,-2,-3,-1, 1,-4,-3,-2,11, 2,-3,-4,-3,-2,-4 ],
						[-2,-2,-2,-3,-2,-1,-2,-3, 2,-1,-1,-2,-1, 3,-3,-2,-2, 2, 7,-1,-3,-2,-1,-4 ],
						[ 0,-3,-3,-3,-1,-2,-2,-3,-3, 3, 1,-2, 1,-1,-2,-2, 0,-3,-1, 4,-3,-2,-1,-4 ],
						[-2,-1, 3, 4,-3, 0, 1,-1, 0,-3,-4, 0,-3,-3,-2, 0,-1,-4,-3,-3, 4, 1,-1,-4 ],
						[-1, 0, 0, 1,-3, 3, 4,-2, 0,-3,-3, 1,-1,-3,-1, 0,-1,-3,-2,-2, 1, 4,-1,-4 ],
						[ 0,-1,-1,-1,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2, 0, 0,-2,-1,-1,-1,-1,-1,-4 ],
						[-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 1 ],
						
						);
			
			
				#  PAM250 source: NCBI
				# This matrix was produced by "pam" Version 1.0.6 [28-Jul-93]
				# PAM 250 substitution matrix, scale = ln(2)/3 = 0.231049
				# Expected score = -0.844, Entropy = 0.354 bits
				# Lowest score = -8, Highest score = 17
				#
			
				my @PAM250 =	(
				
						[ 2,-2, 0, 0,-2, 0, 0, 1,-1,-1,-2,-1,-1,-3, 1, 1, 1,-6,-3, 0, 0, 0, 0,-8 ],
						[-2, 6, 0,-1,-4, 1,-1,-3, 2,-2,-3, 3, 0,-4, 0, 0,-1, 2,-4,-2,-1, 0,-1,-8 ],
						[ 0, 0, 2, 2,-4, 1, 1, 0, 2,-2,-3, 1,-2,-3, 0, 1, 0,-4,-2,-2, 2, 1, 0,-8 ],
						[ 0,-1, 2, 4,-5, 2, 3, 1, 1,-2,-4, 0,-3,-6,-1, 0, 0,-7,-4,-2, 3, 3,-1,-8 ],
						[-2,-4,-4,-5,12,-5,-5,-3,-3,-2,-6,-5,-5,-4,-3, 0,-2,-8, 0,-2,-4,-5,-3,-8 ],
						[ 0, 1, 1, 2,-5, 4, 2,-1, 3,-2,-2, 1,-1,-5, 0,-1,-1,-5,-4,-2, 1, 3,-1,-8 ],
						[ 0,-1, 1, 3,-5, 2, 4, 0, 1,-2,-3, 0,-2,-5,-1, 0, 0,-7,-4,-2, 3, 3,-1,-8 ],
						[ 1,-3, 0, 1,-3,-1, 0, 5,-2,-3,-4,-2,-3,-5, 0, 1, 0,-7,-5,-1, 0, 0,-1,-8 ],
						[-1, 2, 2, 1,-3, 3, 1,-2, 6,-2,-2, 0,-2,-2, 0,-1,-1,-3, 0,-2, 1, 2,-1,-8 ],
						[-1,-2,-2,-2,-2,-2,-2,-3,-2, 5, 2,-2, 2, 1,-2,-1, 0,-5,-1, 4,-2,-2,-1,-8 ],
						[-2,-3,-3,-4,-6,-2,-3,-4,-2, 2, 6,-3, 4, 2,-3,-3,-2,-2,-1, 2,-3,-3,-1,-8 ],
						[-1, 3, 1, 0,-5, 1, 0,-2, 0,-2,-3, 5, 0,-5,-1, 0, 0,-3,-4,-2, 1, 0,-1,-8 ],
						[-1, 0,-2,-3,-5,-1,-2,-3,-2, 2, 4, 0, 6, 0,-2,-2,-1,-4,-2, 2,-2,-2,-1,-8 ],
						[-3,-4,-3,-6,-4,-5,-5,-5,-2, 1, 2,-5, 0, 9,-5,-3,-3, 0, 7,-1,-4,-5,-2,-8 ],
						[ 1, 0, 0,-1,-3, 0,-1, 0, 0,-2,-3,-1,-2,-5, 6, 1, 0,-6,-5,-1,-1, 0,-1,-8 ],
						[ 1, 0, 1, 0, 0,-1, 0, 1,-1,-1,-3, 0,-2,-3, 1, 2, 1,-2,-3,-1, 0, 0, 0,-8 ],
						[ 1,-1, 0, 0,-2,-1, 0, 0,-1, 0,-2, 0,-1,-3, 0, 1, 3,-5,-3, 0, 0,-1, 0,-8 ],
						[-6, 2,-4,-7,-8,-5,-7,-7,-3,-5,-2,-3,-4, 0,-6,-2,-5,17, 0,-6,-5,-6,-4,-8 ],
						[-3,-4,-2,-4, 0,-4,-4,-5, 0,-1,-1,-4,-2, 7,-5,-3,-3, 0,10,-2,-3,-4,-2,-8 ],
						[ 0,-2,-2,-2,-2,-2,-2,-1,-2, 4, 2,-2, 2,-1,-1,-1, 0,-6,-2, 4,-2,-2,-1,-8 ],
						[ 0,-1, 2, 3,-4, 1, 3, 0, 1,-2,-3, 1,-2,-4,-1, 0, 0,-5,-3,-2, 3, 2,-1,-8 ],
						[ 0, 0, 1, 3,-5, 3, 3, 0, 2,-2,-3, 0,-2,-5, 0, 0,-1,-6,-4,-2, 2, 3,-1,-8 ],
						[ 0,-1, 0,-1,-3,-1,-1,-1,-1,-1,-1,-1,-1,-2,-1, 0, 0,-4,-2,-1,-1,-1,-1,-8 ],
						[-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8,-8, 1 ],
						
						);
			
			
				#  PAM500 source: NCBI
				# This matrix was produced by "pam" Version 1.0.6 [28-Jul-93]
				# PAM 500 substitution matrix, scale = ln(2)/7 = 0.0990210
				# Expected score = -0.401, Entropy = 0.0803 bits
				# Lowest score = -9, Highest score = 34
				#
				
				my @PAM500 =	(
						
						[ 1,-1, 0, 1,-2, 0, 1, 1, 0, 0,-1, 0,-1,-3, 1, 1, 1,-6,-3, 0, 1, 0, 0,-9 ],
						[-1, 5, 1, 0,-4, 2, 0,-1, 2,-2,-2, 4, 0,-4, 0, 0, 0, 4,-4,-2, 0, 1, 0,-9 ],
						[ 0, 1, 1, 2,-3, 1, 1, 1, 1,-1,-2, 1,-1,-4, 0, 1, 0,-5,-3,-1, 1, 1, 0,-9 ],
						[ 1, 0, 2, 3,-5, 2, 3, 1, 1,-2,-3, 1,-2,-5, 0, 1, 0,-7,-5,-1, 2, 2, 0,-9 ],
						[-2,-4,-3,-5,22,-5,-5,-3,-4,-2,-6,-5,-5,-3,-2, 0,-2,-9, 2,-2,-4,-5,-2,-9 ],
						[ 0, 2, 1, 2,-5, 2, 2, 0, 2,-1,-2, 1,-1,-4, 1, 0, 0,-5,-4,-1, 2, 2, 0,-9 ],
						[ 1, 0, 1, 3,-5, 2, 3, 1, 1,-2,-3, 1,-1,-5, 0, 1, 0,-7,-5,-1, 2, 2, 0,-9 ],
						[ 1,-1, 1, 1,-3, 0, 1, 4,-1,-2,-3, 0,-2,-5, 1, 1, 1,-8,-5,-1, 1, 1, 0,-9 ],
						[ 0, 2, 1, 1,-4, 2, 1,-1, 4,-2,-2, 1,-1,-2, 0, 0, 0,-2, 0,-2, 1, 2, 0,-9 ],
						[ 0,-2,-1,-2,-2,-1,-2,-2,-2, 3, 4,-2, 3, 2,-1,-1, 0,-5, 0, 3,-2,-2, 0,-9 ],
						[-1,-2,-2,-3,-6,-2,-3,-3,-2, 4, 7,-2, 4, 4,-2,-2,-1,-1, 1, 3,-3,-2,-1,-9 ],
						[ 0, 4, 1, 1,-5, 1, 1, 0, 1,-2,-2, 4, 0,-5, 0, 0, 0,-3,-5,-2, 1, 1, 0,-9 ],
						[-1, 0,-1,-2,-5,-1,-1,-2,-1, 3, 4, 0, 4, 1,-1,-1, 0,-4,-1, 2,-1,-1, 0,-9 ],
						[-3,-4,-4,-5,-3,-4,-5,-5,-2, 2, 4,-5, 1,13,-4,-3,-3, 3,13, 0,-4,-5,-2,-9 ],
						[ 1, 0, 0, 0,-2, 1, 0, 1, 0,-1,-2, 0,-1,-4, 4, 1, 1,-6,-5,-1, 0, 1, 0,-9 ],
						[ 1, 0, 1, 1, 0, 0, 1, 1, 0,-1,-2, 0,-1,-3, 1, 1, 1,-3,-3,-1, 1, 0, 0,-9 ],
						[ 1, 0, 0, 0,-2, 0, 0, 1, 0, 0,-1, 0, 0,-3, 1, 1, 1,-6,-3, 0, 0, 0, 0,-9 ],
						[-6, 4,-5,-7,-9,-5,-7,-8,-2,-5,-1,-3,-4, 3,-6,-3,-6,34, 2,-6,-6,-6,-4,-9 ],
						[-3,-4,-3,-5, 2,-4,-5,-5, 0, 0, 1,-5,-1,13,-5,-3,-3, 2,15,-1,-4,-4,-2,-9 ],
						[ 0,-2,-1,-1,-2,-1,-1,-1,-2, 3, 3,-2, 2, 0,-1,-1, 0,-6,-1, 3,-1,-1, 0,-9 ],
						[ 1, 0, 1, 2,-4, 2, 2, 1, 1,-2,-3, 1,-1,-4, 0, 1, 0,-6,-4,-1, 2, 2, 0,-9 ],
						[ 0, 1, 1, 2,-5, 2, 2, 1, 2,-2,-2, 1,-1,-5, 1, 0, 0,-6,-4,-1, 2, 2, 0,-9 ],
						[ 0, 0, 0, 0,-2, 0, 0, 0, 0, 0,-1, 0, 0,-2, 0, 0, 0,-4,-2, 0, 0, 0, 0,-9 ],
						[-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9,-9, 1 ],

						);
				
				my @MATCH =	(
						
						[ 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,13,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,13,-1,-1,-1,-1, 1,-1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1 ],
						[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1 ],

						);
				
				
=pod

Following lines generate a hash with scoring values for a given scoring matrix and fill indel scoring with scoring of stop codons derived from the scoring matrix.


=cut
				my @aminoacids = ( 'A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'B', 'Z', 'X', '*');
			
				my $create = sub {
					
							my $aa_lead    = 0 ;
				
							for my $line (@_) {
								
									for (my $a=0;$a<=$#{$line};$a++) {
					
										$SCORING{$aminoacids[$aa_lead].$aminoacids[$a]} = $line -> [$a];
				
									}
				
									$aa_lead ++
							}
					
						} ;
				
				
				
				$matrix eq 'BLOSUM62' and do { $create -> ( @BLOSUM62 ) } ;
				$matrix eq 'PAM250'   and do { $create -> ( @PAM250   ) } ;
				$matrix eq 'PAM500'   and do { $create -> ( @PAM500   ) } ;
				$matrix eq 'MATCH'    and do { $create -> ( @MATCH    ) } ;
				
				my $indel_score =  $SCORING{'A'.'*'};
				
				for (@aminoacids) { $SCORING{$_.'-'}  = $indel_score; $SCORING{'-'.$_}  = $indel_score; $SCORING{'-'.'-'} = $indel_score }
					
					    
			
			};

			return \%SCORING;
	
	}#end of sub get_scoring


#_______________________________________________________________________________________


sub ambiguity_table {
	
			my %TABLE;
			my @nucleotides = qw ( A C G T - M R W S Y K V H B D N A- C- G- T- M- R- W- S- Y- K- V- H- B- D- N-);
			my %ambiguities =    ('A'=>'A','C'=>'C','G'=>'G','T'=>'T','-'=>'-','A-'=>'A-','C-'=>'C-','G-'=>'G-','T-'=>'T-','AC'=>'M','AG'=>'R','AT'=>'W',
					      'CG'=>'S','CT'=>'Y','GT'=>'K','ACG'=>'V','ACT'=>'H','CGT'=>'B','AGT'=>'D','ACGT'=>'N','AC-'=>'M-','AG-'=>'R-','AT-'=>'W-',
					      'CG-'=>'S-','CT-'=>'Y-','GT-'=>'K-','ACG-'=>'V-','ACT-'=>'H-','CGT-'=>'B-','AGT-'=>'D-','ACGT-'=>'N-');
	
	
			for my $i (@nucleotides){
			    my ($recode1,$recode2);
		
				for my $key (keys %ambiguities){
					if ($i eq $ambiguities{$key}){ $recode1 = $key}
					}
		
			for my $j (@nucleotides){
				
				for my $key (keys %ambiguities){
					if ($j eq $ambiguities{$key}){ $recode2 = $key}
					}	
			
				my @merging = sort (split"",($recode1.$recode2));
			
				my $merging =   join"",@merging;
				   $merging =~ s/(-*)(.+)/$2$1/;
				   $merging =~ tr/ACGT-/ACGT-/s;
					       	    
			      $TABLE{$i.$j} = $ambiguities{$merging};
	       		
					}
				}

			return \%TABLE;
	}#end of sub ambiguity table	   

#_______________________________________________________________________________________

#sub of getting threshhold fot aa scoring

sub get_threshhold {
	
			my ($ref_scoring,$window,$ref_FASTA) = @_;
			my @aminoacids = qw(A R N D C Q E G H I L K M F P S T W Y V);
			my %freq_aa;
			my $aa_total;
			my @freq_aa_base;
			my %freq_scores;
	
			for my $ref_sequence (values %$ref_FASTA) { $aa_total += (grep /[ARNDCQEGHILKMFPSTWYV]/,@$ref_sequence ) }
		
	
			for my $aa (@aminoacids) {
				my $aa_single;
				for my $ref_sequence (values %$ref_FASTA) { $aa_single += (grep /$aa/,@$ref_sequence ) }
				$freq_aa{$aa} = int (($aa_single/$aa_total) * 100 )
		
			}
		
		
			for my $key (keys %freq_aa) {
				push @freq_aa_base, ($key) x ( $freq_aa {$key}*10 )
	
			}
		
		
			for (1..100) {
				
				my @boot;
				push @boot , \$freq_aa_base[int(rand($#freq_aa_base + 1))] until (($#freq_aa_base + 1) == @boot);
				
				
					for (1..100)  {
						my @half_boot;
						push @half_boot , $boot[int(rand($#boot + 1))] until (($#boot + 1)/2 == @half_boot);	
						
						for (1..100) {
							my $score;
					
								for (1..$window) {
									$score += $$ref_scoring { ${$half_boot[ int(rand($#half_boot + 1))]}.${$half_boot[ int(rand($#half_boot + 1))]} }
								}
					
						$freq_scores {$score} ++
						}
					}	
			}
			
			map { $_/=(100*100*100) } (values %freq_scores);
		
			my  ($cum_score,$threshhold);	
			
			#open OUT,">","distribution.txt";
			
			CUTOFF:	for my $single_score (sort {$a<=>$b} keys %freq_scores){
					
			#		print OUT $single_score,"\t",$freq_scores {$single_score},"\n";
					
					$cum_score                 += $freq_scores{$single_score};
					$freq_scores{$single_score} = $cum_score;
					$threshhold                 = $single_score and last CUTOFF if $freq_scores{$single_score} >= 0.95
				}
		#print $threshhold,"\n";
	return $threshhold;
	
	};

#_______________________________________________________________________________________

#sub replace ambiguities in DNA, sequences must be 2-dimensional arrays

sub rp_ambiguities {
			my ($position) = @_;
			#ambiguities
			my @N = qw(A C G T);
			my @M = qw(A C);
			my @R = qw(A G);
			my @W = qw(A T);
			my @S = qw(C G);
			my @Y = qw(C T);
			my @K = qw(G T);
			my @V = qw(A C G);
			my @H = qw(A C T);
			my @B = qw(C G T);
			my @D = qw(A G T);
			#replace with arrays
				if (grep /N/,@$position){shift @$position;push @$position, @N}
				if (grep /M/,@$position){shift @$position;push @$position, @M}
				if (grep /R/,@$position){shift @$position;push @$position, @R}
				if (grep /W/,@$position){shift @$position;push @$position, @W}
				if (grep /S/,@$position){shift @$position;push @$position, @S}
				if (grep /Y/,@$position){shift @$position;push @$position, @Y}
				if (grep /K/,@$position){shift @$position;push @$position, @K}
				if (grep /V/,@$position){shift @$position;push @$position, @V}
				if (grep /H/,@$position){shift @$position;push @$position, @H}
				if (grep /B/,@$position){shift @$position;push @$position, @B}
				if (grep /D/,@$position){shift @$position;push @$position, @D}	
	}#end of sub rp_ambiguities


#_______________________________________________________________________________________


sub nuc_score_two {
	
			my ($ref_scoring,$variability,$window,$seq1,$seq2) = @_;
			
			#sequences are read as references, $seq1, $seq2 are references	
			my @basket = ($seq1,$seq2);
			
			#check whether sequences are of equal length
			die "sequences are not of equal length!\n" if (my $length1 = @$seq1)!= (my $length2 = @$seq2);
			my $length = $length1;
			
			#declare and initialize return array of alignment quality scores and references to input sequences
			my @MSA_Score = (0)x $length;

			#start sliding window approach for MC quality score, labeled SLIDE
			SLIDE:	foreach my $j (@$variability){
			
				my ($MC_score,%MC_Hash,$score,$threshhold,$Org_score);
				my $position = $j;

			#evaluates window of original sequences
					$Org_score += $$ref_scoring{$seq1->[$position].$seq2->[$position]}, $position++ for (1..$window);#print $Org_score,"\n";exit;
					undef $position;
			#print "org_score: $Org_score, position: $j\n";exit;

			#checks $Org_score value, starts MC resampling if Org_score did not hit max or min value
			#PERFECT:
			if ($Org_score == $window){
					map {$_+=1} @MSA_Score[$j..($j+($window-1))] and next SLIDE;#print "@MSA_Score\n";
					}
			#NOTHING:
			elsif ($Org_score == (0-$window)){
					map {$_-=1} @MSA_Score[$j..($j+($window-1))] and next SLIDE;#print "@MSA_Score\n";
					}
			#starts MC resampling, since Org_score did not hit max or min value	
			#TESTING:
			else {
			#set MC sampling pool from a double size sequence window,
			#does not move compositional window until $j>=$window/2
			#LEFT:
					if ($j < ($window/2)){
			#runs the MC resampling approach to generate new similarity scores of randomly drawn sequences
						foreach (1..100){
							foreach (1..$window){
								$MC_score=0;
			#selects randomly two positions from compositional window of both sequences
								$MC_score+=$$ref_scoring{$basket[int(rand(2))]->[int(rand($window*2))].$basket[int(rand(2))]->[int(rand($window*2))]};
								}
			#print "mc-score: $MC_score\n";
							$MC_Hash{$MC_score}++;
		      					}
						}	   
			#does not move compositional window if $j>$length-($window*2+1)	
			#RIGHT:
					elsif ($j > ($length-($window+$window/2))){
			#runs the MC resampling approach to generate new similarity scores of randomly drawn sequences
						foreach (1..100){
							foreach (1..$window){
								$MC_score=0;
								$MC_score+=$$ref_scoring{$basket[int(rand(2))]->[$length-(int(rand($window*2))+1)].$basket[int(rand(2))]->[$length-(int(rand($window*2))+1)]};
								}
							$MC_Hash{$MC_score}++;
		      					}
						}	
			#slides compositional window if ($window/2)<$j>$length-($window*2+1)	
			#MIDDLE:
					else {
			#runs the MC resampling approach to generate new similarity scores of randomly drawn sequences
						foreach (1..100){
							foreach (1..$window){
								$MC_score=0;
								$MC_score+=$$ref_scoring{$basket[int(rand(2))]->[(int(rand($window*2)))+($j-$window/2)].$basket[int(rand(2))]->[(int(rand($window*2)))+($j-$window/2)]};	    
								}
							$MC_Hash{$MC_score}++;
		      					}
						}	
			#MC resampling finished, Hash with scores of random sampling generated

			#summarizes hits of MC resampling
			CUTOFF:		foreach $MC_score (sort {$a<=>$b} keys %MC_Hash){
			#print "key: $MC_score\t->value: $MC_Hash{$MC_score}\t";
						$score+=$MC_Hash{$MC_score};
						$MC_Hash{$MC_score}=$score;#print "cumul. score: $score\n";
						$threshhold=$MC_score and last CUTOFF if $MC_Hash{$MC_score}>=95
						}
						
			#sets score value for each position in MSA Aligroove array
			#print "threshhold: $threshhold\n";
						map {$_+=1} @MSA_Score[$j..($j+($window-1))] if $Org_score > $threshhold;	
						map {$_-=1} @MSA_Score[$j..($j+($window-1))] if $Org_score <= $threshhold;
			#print "@MSA_Score\n";
			
				}#end of else TESTING
				
				
			#cleaning of runing variables
			($Org_score,$MC_score,$threshhold)=();
			undef %MC_Hash;
			
			}#end of sliding window and for loop

			return \@MSA_Score;

	} #end of sub pars_score_two


#_______________________________________________________________________________________



sub aa_score_two {
	
			my ($ref_scoring,$threshhold,$window,$seq1,$seq2) = @_;
			
			#sequences are read as references, $seq1, $seq2 are references	
			my @basket = ($seq1,$seq2);
			
			#check whether sequences are of equal length
			die "sequences are not of equal length!\n" if (my $length1 = @$seq1) != (my $length2 = @$seq2);
			my $length = $length1;
			
			#declare and initialize return array of alignment quality scores and references to input sequences
			my @MSA_Score = (0)x $length;
 
			
			#start sliding window approach for MC quality score, labeled SLIDE
			SLIDE:	foreach my $j (0..$length-($window+1)){
			
					my $Org_score    ;
					my $position = $j;

			#evaluates window of original sequences
			
					$Org_score += $$ref_scoring{$seq1->[$position].$seq2->[$position]}, $position++ for (1..$window);#print $Org_score,"\n";exit;
					
			#print "org_score: $Org_score, position: $j\n";#exit;

			
			#PERFECT:
					if ($Org_score  > $threshhold) {
						map {$_+=1} @MSA_Score[$j..($j+($window-1))] and next SLIDE;
					}
			#NOTHING:
					if ($Org_score <= $threshhold) {
						map {$_-=1} @MSA_Score[$j..($j+($window-1))] and next SLIDE;#print "@MSA_Score\n";
					}
			
				
				}#end of else SLIDE
				
	return \@MSA_Score;

	} #end of sub pars_score_two


#_______________________________________________________________________________________


#sub for overlap of arrays

sub overlap {
			my ($arr1,$arr2)=@_;
			my (%hash2,@overlap);

			$hash2{$_}=1 for (@$arr2);
			for (0..$#$arr1){push @overlap, $arr1->[$_] if (defined $hash2{$arr1->[$_]})};
	
			my $hit=@overlap;
			return \$hit;
	}	
#_______________________________________________________________________________________


#sub read FASTA file, returns references to fasta hash, keys taxa, values sequences

sub readFASTA_simple {
	

		my ($taxon,@FASTA,%FASTA,%STRUCTURES,$number,$type,$aa_type);
		
		$aa_type             = 0 ;

		my ( $file ) = @_;
		
		
		#makes sure that linefeeds are just naked \n and nothing else
			
			(open IN , $file ) || die "file $file can not be opened!\n"; close IN ;
			
			(tie ( my @data, 'Tie::File', $file ));
			
			die "file $file is empty!\n" if 0 == @data ;
			map { s/\r\n/\n/g } @data ;
			map { s/\r/\n/g   } @data ;

			untie @data ;
		
		open (my $fh, "<",$file) || die "file $file not found! terminated\n";
	
		READING: 
		
		while ( <$fh> ) {

			die "convert from mac format!\n" if /\r[^\n]/;
		
			chomp ;
				
			next READING                if /^\s+$/;
			die "taxon name missing!\n" if /^\>$/ ;
			
			if ( /\>/ ) {		
			
				$taxon = $_                                               ;
				$taxon =~ s/^\>\s*/\>/g                                   ;
				$taxon =~ s/\s*$//g                                       ;
				die "multiple taxa of $taxon\n" if defined $FASTA{$taxon} ; 
				next READING                                              ;
				
			}
	
			s/\s*//g and $FASTA{$taxon} .= $_ ;
		
		}
		
	
		for my $key ( keys %FASTA ) { 
			
			my $NAMEERROR = do { "forbidden sign or space in\n$key\n" } ;
			
			for ( $key ) {
				
				die $NAMEERROR if /\|/ ; 
				die $NAMEERROR if /\(/ ;
				die $NAMEERROR if /\)/ ;
				die $NAMEERROR if /\:/ ;
				die $NAMEERROR if /\;/ ;
				die $NAMEERROR if /\,/ ;
				die $NAMEERROR if /--/ ;
				die $NAMEERROR if /\*/ ;
				die $NAMEERROR if /\s/ ;
				
			}
			
			die "file is not in FASTA format\n" if $key          !~  /^\>/;
			die "file is not in FASTA format\n" if $FASTA{$key}  =~  /\>/ ;
			die "not enough sequences!\n"       if 1>= keys %FASTA        ;
			
			my %seen = () ;
			
			my @TAXA = grep { ! $seen{$_}++ } keys %FASTA ;

		}
		
	
	#cleaning of taxa and sequences removing leading and trailing blanks
	
		grep s/^\>// || s/^\s*// || s/\s*$// , @FASTA = %FASTA;
		
		%FASTA = @FASTA;
		
		
	#identifying structure strings, cleaning, reading and modifying sequences using input options	
			
		for $taxon ( keys %FASTA ) {
			
			no warnings ;
			
			if ( $FASTA{$taxon} =~ /\(/ && /\)/ && /\./ ) {
				
				s/ //g;
				
				die "structure not balanced\n" if grep /\(/,$FASTA{$taxon} != grep /\)/,$FASTA{$taxon};
				
				print "\tstructure: $taxon \-\> ",length ( $FASTA{$taxon} ),"\n";
				
				$STRUCTURES{$taxon} = $FASTA{$taxon};
				
				delete $FASTA{$taxon}
			}
			
			else {
				
				$number ++;
				
				$FASTA{$taxon} = uc( $FASTA{$taxon} );
			
				$FASTA{$taxon} =~ s/ //g;		#print $taxon,"\n";
			
				die "\nsequence $taxon contains unknown characters!\nprocess terminated!\n" if $FASTA{$taxon} =~ /[^A-Z\?\-\*]/;	
				
				my $abs_indelN = ($FASTA{$taxon} =~ tr/N\-//      );
				
				if ( length( $FASTA{$taxon} ) != $abs_indelN ) {
					
					my $freq_ACGT  = ($FASTA{$taxon} =~ tr/ACG(T|U)// )/(length( $FASTA{$taxon} ) - $abs_indelN);
				
					   $type       =  $freq_ACGT > 0.8 ? "nt" : "aa";
				}
				
				if ( length( $FASTA{$taxon} ) == $abs_indelN ) {
					
					$number --
				}
				
				my @sequence   = split"", $FASTA{$taxon} ;
		
					$type eq "nt" and do {
						
						grep { s/\?/N/ } @sequence ;
						grep { s/\*/N/ } @sequence
					};

					$type eq "aa" and do {
						
						grep { s/\?/-/ } @sequence ;
					};

	#reports on sequence length
				
				printf     "\t%-20.20s \-\> sites: %-6s\n" , $taxon, $#sequence+1;
		
				$FASTA{$taxon} = \@sequence;
		
				$aa_type++ if $type eq "aa";
		
				}
				
		}
		
	die "mixture of nucleotide and amino acid sequences!\n" if (0 < ($number - $aa_type)/$number) && (1 > ($number - $aa_type)/$number) ;
	
	#returns reference to FASTA hash and structure hash		

	return (\%FASTA,\%STRUCTURES,$type);print "\n\n";#exit;
	
	}#end of sub
#_______________________________________________________________________________________









1;











